<?php

/**
 * WeEngine Team
 *
 * (c) We7Team 2019 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

use W7Store\lib\lib_utils;

if (!defined('IN_DISCUZ')) {
	exit('Access Denied');
}

require_once DISCUZ_ROOT . '/source/plugin/w7store/config/index.php';
require_once libfile('lib/apps', 'plugin/' . W7_STORE);

global $_G;

$apps = new \W7Store\lib\lib_apps($config);
$act = isset($_GET['act']) ? $_GET['act'] : '';
$serverUrl = 'action=plugins&operation=config&do=' . $pluginid . '&identifier=' . W7_STORE . '&pmod=apps';
if (!$apps->isEnable()) {
	cpmsg_error($apps->getErrorMessage(), 'action=plugins');
}
$apps->checkAccessTokenExpires();

if ($act == 'sync') {
	$result = $apps->sync();
	if (!$result) {
		cpmsg_error($apps->getErrorMessage(), $serverUrl);
	}

	cpmsg('数据同步成功', $serverUrl);
} elseif (in_array($act, ['build', 'upgrade', 'branch'])) {
	$pluginName = isset($_GET['pluginName']) ? $_GET['pluginName'] : '';
	if (!$pluginName) {
		cpmsg_error('参数错误', $serverUrl);
	}

	$stepUrl = $serverUrl . "&pluginName={$pluginName}&act={$act}";
	$step = isset($_GET['step']) ? $_GET['step'] : 0;
	if ($step == 1) {
		$apps->setSiteInfo();
		$result = $apps->build($pluginName);
		if (!$result) {
			cpmsg_error($apps->getErrorMessage(), $serverUrl);
		}

		$errors = [];
		foreach ($result['files'] as $file) {
			$progress = $start / $total;
			if (!$apps->download($file['path'], $file['checksum'], $pluginName)) {
				$errors[] = $file;
			}
		}

		if (empty($errors)) {
			$apps->doBuild($pluginName, $result['files'], $result['manifest']);
			cpmsg('cloudaddons_downloading', $stepUrl . '&step=2', 'loading', array('addonid' => $pluginName . '.plugin'), '<div>100%</div>', false);
		} else {
			cpmsg_error('安装失败', $serverUrl);
		}
	} elseif ($step == 2) {
		$url = 'action=plugins&operation=import&dir=' . $pluginName;
		// 如果操作是更新/新分支,检查插件是否安装,如果已安装就跳转到更新操作,否则就安装插件
		if ($act == 'upgrade') {
			$apps->sync();
			$plugin = C::t('common_plugin')->fetch_by_identifier($pluginName);
			if ($plugin) {
				$url = 'action=plugins&operation=upgrade&pluginid=' . $plugin['pluginid'];
			}
		}
		cpmsg('cloudaddons_installing', $url, 'loading', array('addonid' => $pluginName . '.plugin'), '<div>100%</div>', false);
	} else {
		cpmsg('cloudaddons_downloading', $stepUrl . '&step=1', 'loading', array('addonid' => $pluginName . '.plugin'), '<div>0%</div>', false);
	}
} elseif ($act == 'delete') {
	$pluginId = isset($_GET['pluginId']) ? intval($_GET['pluginId']) : 0;
	$plugin = C::t($config['table_apps'])->fetch($pluginId);
	if (!$plugin) {
		cpmsg_error('插件未安装/已删除', $serverUrl);
	}

	C::t($config['table_apps'])->update_by_identifier($plugin['identifier'], ['status' => 0]);
	C::t($config['table_install'])->remove($plugin['identifier']);
	$doPlugin = C::t('common_plugin')->fetch_by_identifier($plugin['identifier']);
	if ($doPlugin) {
		dheader("Location: {$_G['siteurl']}/admin.php?action=plugins&operation=delete&pluginid={$doPlugin['pluginid']}");
	} else {
		$utils = new lib_utils();
		$utils->dirClear(DISCUZ_ROOT . 'source/plugin/' . $plugin['identifier']);
		cpmsg('plugins_delete_succeed', 'action=plugins', 'succeed');
	}
} else {
	// 是否需要同步应用
	$setting = C::t($config['table_setting'])->info();
	if ((time() - $setting['sync_time']) > $config['apps_sync_time']) {
		$apps->sync();
	}

	$page = isset($_GET['page']) ? intval($_GET['page']) : 1;
	$page = max(1, $page);
	$limit = 20;
	$table = DB::table(C::t($config['table_apps'])->getTable());
	$total = DB::result_first('SELECT count(*) FROM ' . $table);
	if ($total > 0) {
		$totalNumber = lang($config['lang'], 'total_number');
		$totalPageNumber = $total / $limit;
		$totalPageNumber = @ceil($totalPageNumber);
		$totalNumber = str_replace('x', $totalPageNumber, $totalNumber);
		$totalNumber = str_replace('y', $total, $totalNumber);
		$totalNumber = str_replace('z', $page, $totalNumber);
		$page = min($totalPageNumber, $page);
		$startNum = ($page - 1) * $limit;
		$homePage = $serverUrl . '&page=1';
		$nextPage = $serverUrl . '&page=' . ($page + 1);
		$prePage = $serverUrl . '&page=' . ($page - 1);
		$endPage = $serverUrl . '&page=' . $totalPageNumber;
		$lists = DB::fetch_all("SELECT * FROM {$table} WHERE 1 ORDER BY id DESC LIMIT {$startNum},{$limit}");
		foreach ($lists as $key => $val) {
			$val['store_url'] = $apps->getStoreUrl($val['id']);
			$val['site_branch'] = json_decode($val['site_branch'], true);
			$val['support'] = $apps->getSupportBranch($val['site_branch']);
			$lists[$key] = $val;
		}
		$start = $page - 4;
		$start = min($totalPageNumber - 8, $start);
		$start = max($start, 1);
		$end = $start + 8;
		$end = min($end, $totalPageNumber);
		$showPage = '';
		for ($i = $start; $i <= $end; $i++) {
			if ($i == $page) {
				$showPage = $showPage . '<strong>' . $i . '</strong>';
			} else {
				$showPage = $showPage . '<a href="?' . $serverUrl . '&page=' . $i . '&formhash=' . FORMHASH . '">' . $i . '</a>';
			}
		}
	}

	$storeUrl = $config['store_host'];
	include template(W7_STORE . ':apps');
}
