<?php

/**
 * WeEngine Team
 *
 * (c) We7Team 2019 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7Store\lib;

if (!defined('IN_DISCUZ')) {
	exit('Access Denied');
}

require_once libfile('lib/auth', 'plugin/' . W7_STORE);
require_once libfile('lib/utils', 'plugin/' . W7_STORE);
require_once libfile('lib/touch', 'plugin/' . W7_STORE);

class lib_apps extends lib_auth {
	protected $siteInfo = null;

	public function __construct($config = []) {
		parent::__construct($config);
	}

	public function getSiteInfo() {
		return $this->siteInfo;
	}

	public function setSiteInfo() {
		global $_G;

		$setting = \C::t($this->config['table_setting'])->info();
		// 没有获取到Token,跳转到设置站点信息页面
		if (empty($setting['site_token'])) {
			dheader('Location: ' . $_G['siteurl'] . 'admin.php?action=plugins&identifier=' . W7_STORE . '&pmod=setting');
		}

		$site = $setting['remark']['site'];
		$parseUrl = parse_url($site['url']);
		$siteInfo = new \W7\Message\SiteInfo();
		$siteInfo->setKey($site['key']);
		$siteInfo->setFamily($site['family']);
		$siteInfo->setHost($parseUrl['host']);
		$siteInfo->setRelease($site['release']);
		$siteInfo->setToken($site['token']);
		$siteInfo->setVersion($site['version']);
		$this->siteInfo = $siteInfo;
	}

	public function getApps() {
		try {
			$cache = new \W7\Cache\File($this->config['plugin_cache_path']);
			$moduleQuery = new \W7\Api\Module\Query($cache);
			return $moduleQuery->setSiteInfo($this->siteInfo)->get();
		} catch (\Exception $exception) {
			$this->setErrorMessage($exception->getMessage());
			return false;
		}
	}

	public function sync() {
		$this->setSiteInfo();
		$apps = $this->getApps();
		if (!$apps) {
            $this->setErrorMessage('没有同步到数据');
			return false;
		}

		// 系统已安装的应用
		$installPlugins = \C::t('common_plugin')->fetch_all_data();
		$pluginIdentifiers = array_column($installPlugins, 'identifier');

		// 应用中心已安装的应用
		$doInstallPlugins = \C::t($this->config['table_install'])->fetch_all();
		$doInstallPluginKeys = array_column($doInstallPlugins, 'plugin_identifier');
		$pluginIdentifiers = array_unique(array_merge($pluginIdentifiers, $doInstallPluginKeys));
		$doInstallPlugins = array_combine($doInstallPluginKeys, $doInstallPlugins);
		$doPluginVersions = array_column($doInstallPlugins, 'plugin_version', 'plugin_identifier');

		$table = \DB::table(\C::t($this->config['table_apps'])->getTable());
		$sql = "TRUNCATE TABLE {$table}";
		\DB::query($sql);

		$chunk = array_chunk($apps, $this->config['table_insert_max']);
		$fields = 'id,name,identifier,author,logo,version,status,displayorder,has_new_version,has_new_branch,main_module,branch_id,site_branch,branches,description,buy_time,created_time';
		$statement = "INSERT INTO {$table} ({$fields}) VALUES ";
		$now = time();

		foreach ($chunk as $tmpApps) {
			$rows = [];
			foreach ($tmpApps as $val) {
				$siteBranch = json_encode($val['site_branch'], JSON_UNESCAPED_UNICODE);
				$branches = json_encode($val['branches'], JSON_UNESCAPED_UNICODE);
				$status = in_array($val['name'], $pluginIdentifiers) ? 1 : 0;
				$hasNewVersion = 0;
				$hasNewBranch = 0;
				if ($status) {
					if (isset($doPluginVersions[$val['name']]) && version_compare($doPluginVersions[$val['name']], $val['version'], '<')) {
						$hasNewVersion = 1;
					}
					$lastBranches = end($val['branches']);
					if (in_array($val['name'], $doInstallPluginKeys)) {
						if ($lastBranches['displayorder'] > $doInstallPlugins[$val['name']]['plugin_branch_displayorder']) {
							$hasNewBranch = 1;
						}
					}
				}
				$rows[] = "('{$val['id']}', '{$val['title']}', '{$val['name']}', '{$val['author']}', '{$val['thumb']}', '{$val['version']}', '{$status}', '{$val['displayorder']}', '{$hasNewVersion}', '{$hasNewBranch}', '{$val['main_module']}', '{$val['branch']}', '{$siteBranch}', '{$branches}', '{$val['description']}', '{$val['buytime']}', '{$now}')";
			}
			$rows = implode(',', $rows);
			$sql = $statement . $rows;
			\DB::query($sql);
		}

		\C::t($this->config['table_setting'])->updateSite(['sync_time' => $now]);
		return true;
	}

	public function getStoreUrl($pluginId) {
		return $this->config['store_host'] . '/module-' . $pluginId . '.html';
	}

	public function build($identifier) {
		try {
			$cache = new \W7\Cache\File($this->config['plugin_cache_path']);
			$moduleInfo = new \W7\Api\Module\Info($cache);
			$info = $moduleInfo->setSiteInfo($this->siteInfo)->setName($identifier)->get();
			$module = new \W7\Api\Module\Build($cache);
			return $module->setSiteInfo($this->siteInfo)->setVersion($info['version']['version'])->setName($identifier)->get();
		} catch (\Exception $exception) {
			$this->setErrorMessage($exception->getMessage());
			return false;
		}
	}

	public function doBuild($identifier, $files, $xml) {
		// addonmd5
		$md5file = $identifier.'.plugin';
		$end = "Status=End&ID={$md5file}&SN=w7sn&RevisionID=0&RevisionDateline=" .time();
		$md5 = $this->handleMd5Files($files);
		cloudaddons_savemd5($md5file, $end, $md5);

		// xml
		file_put_contents(DISCUZ_ROOT . 'source/plugin/' . $identifier . '/discuz_plugin_' . $identifier . '.xml', $xml);

		$plugin = \C::t($this->config['table_apps'])->fetch_by_identifier($identifier);
		\C::t($this->config['table_install'])->saveSite($plugin['id'], $plugin['identifier'], $plugin['version'], $plugin['site_branch']['id'], $plugin['site_branch']['displayorder']);
		return \C::t($this->config['table_apps'])->update_by_identifier($identifier, ['status' => 1]);
	}

	protected function handleMd5Files($files) {
		$data = [];
		foreach ($files as $val) {
			$data[$val['path']] = $val['checksum'];
		}
		return $data;
	}

	public function download($path, $validate, $identifier) {
		try {
			$path = '/' . $identifier . $path;
			$filename = $this->getFilename($path);

			// 如果文件存在,则不需要下载
			if ($this->checkFileExists($filename, $validate)) {
				return true;
			}

			$cache = new \W7\Cache\File($this->config['plugin_cache_path']);
			$download = new \W7\Api\Common\Download($cache);
			$result = $download->setSiteInfo($this->siteInfo)->setPath($path)->get();
			if (!$result) {
				$this->setErrorMessage('文件的内容错误: ' . $path);
				return false;
			}
			if (isset($result['path']) && isset($result['file'])) {
				$this->handleDownload($result);
			}
			return true;
		} catch (\Exception $exception) {
			$this->setErrorMessage($exception->getMessage());
			return false;
		}
	}

	public function handleDownload($data) {
		$filename = $this->getFilename($data['path']);
		$touch = new lib_touch();
		return $touch->saveFile($filename, $data['file']);
	}

	public function getFilename($path, $filter = '') {
		return DISCUZ_ROOT . $path;
	}

	public function checkFileExists($filename, $checkSum) {
		if (file_exists($filename) && md5(file_get_contents($filename)) == $checkSum) {
			return true;
		}
		return false;
	}

	/**
	 * 获取支持的分支
	 * @param $branch
	 * @return array
	 */
	public function getSupportBranch($branch) {
		$support = [
			'app' => [
				'name' => '公众号'
			],
			'wxapp' => [
				'name' => '小程序'
			],
			'android' => [
				'name' => '安卓'
			],
			'ios' => [
				'name' => '苹果'
			],
			'webapp' => [
				'name' => 'PC应用'
			],
		];

		$supportApp = array_keys($support);
		$data = [];
		foreach ($branch as $key => $val) {
			if (stripos($key, 'support') !== false && $val == 2) {
				$app = str_replace('_support', '', $key);
				if (in_array($app, $supportApp)) {
					$data[] = $support[$app];
				}
			}
		}

		return $data;
	}
}
