<?php

/**
 * WeEngine Team
 *
 * (c) We7Team 2019 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7Store\lib;

if (!defined('IN_DISCUZ')) {
	exit('Access Denied');
}

require_once libfile('lib/base', 'plugin/' . W7_STORE);

class lib_auth extends lib_base {
	public function __construct($config = []) {
		parent::__construct($config);
	}

	public function checkAccessToken() {
		$setting = \C::t($this->config['table_setting'])->info();
		if ($setting && $setting['access_token'] != '' && $setting['access_token_expires'] > time()) {
			return true;
		}
		return false;
	}

	public function checkAccessTokenExpires() {
		global $_G;

		if (!$this->checkAccessToken()) {
			$url = $_G['siteurl'] . 'plugin.php?id=w7store:login';
			dheader('location:' . $url);
		}
	}

	public function getLoginUrl($redirect) {
		try {
			$auth = new \W7\Api\Oauth\LoginUrl();
			$auth->getByDiscuz(urlencode($redirect));
			return json_decode($auth->getRequestContent(), true);
		} catch (\Exception $exception) {
			$this->setErrorMessage($exception->getMessage());
			return false;
		}
	}

	public function getAccessToken($code) {
		try {
			$auth = new \W7\Api\Oauth\AccessToken();
			$auth->get($code);
			return json_decode($auth->getRequestContent(), true);
		} catch (\Exception $exception) {
			$this->setErrorMessage($exception->getMessage());
			return false;
		}
	}

	public function getUser($accessToken) {
        try {
            $user = new \W7\Api\Oauth\User();
            $user->setAccessToken($accessToken);
            $user->get();
            return json_decode($user->getRequestContent(), true);
        } catch (\Exception $exception) {
            $this->setErrorMessage($exception->getMessage());
            return false;
        }
    }
}
