<?php

/**
 * WeEngine Cloud SDK System
 *
 * (c) We7Team 2019 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Message;

final class SiteInfo {
	/**
	 * @var 站点id
	 */
	private $key;

	/**
	 * @var 站点Host
	 */
	private $host;

	/**
	 * @var 当前站点版本类型，商业版，普通版
	 */
	private $family;

	/**
	 * @var 当前站点token
	 */
	private $token;

	/**
	 * @var 当前站点通信密码
	 */
	private $password;

	/**
	 * @var 当前请求接口操作类型
	 */
	private $method;

	/**
	 * @var 当前PHP版本号
	 */
	private $php_version;

	/**
	 * @var 当前host
	 */
	private $current_host;

	/**
	 * @var 当前发行版本号
	 */
	private $release;

	/**
	 * @var 站点类型: 简体 繁体
	 */
	private $locale;

	/**
	 * @var 用于接收数据时保存的临时文件的键名
	 */
	private $file;

	public function toArray() {
		if (empty($this->key) || empty($this->token) || empty($this->host)) {
			throw new \RuntimeException('站点信息不完整');
		}

		$info = array(
			'key' => $this->key,
			'host' => $this->host,
			'method' => $this->method,
			'family' => $this->family,
			'version' => $this->version,
			'release' => $this->release,
			'password' => md5($this->key . $this->token),
			'php_version' => PHP_VERSION,
			'current_host' => strpos($_SERVER['HTTP_HOST'], ':') !== false ? parse_url($_SERVER['HTTP_HOST'], PHP_URL_HOST) : $_SERVER['HTTP_HOST'],
		);

		if (!empty($this->file)) {
			$info['file'] = $this->file;
		}

		return $info;
	}

	/**
	 * @param 站点id $key
	 */
	public function setKey($key) {
		$this->key = $key;
	}

	/**
	 * @param 当前站点版本类型，商业版，普通版 $family
	 */
	public function setFamily($family) {
		$this->family = $family;
	}

	/**
	 * @param 当前发行版本号 $release
	 */
	public function setRelease($release) {
		$this->release = $release;
	}

	/**
	 * @param 站点Host $host
	 */
	public function setHost($host) {
		$this->host = $host;
	}

	/**
	 * @param 当前站点token $token
	 */
	public function setToken($token) {
		$this->token = $token;
	}

	/**
	 * @param mixed $version
	 */
	public function setVersion($version) {
		$this->version = $version;
	}

	/**
	 * @param 当前请求接口操作类型 $method
	 */
	public function setMethod($method) {
		$this->method = $method;
	}
}
