<?php

/**
 * WeEngine Cloud SDK System
 *
 * (c) We7Team 2019 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Util;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use W7\Cache\CacheInterface;
use W7\Exception\SiteRegisteredException;

!defined('W7_CLOUD_SDK_DEVELOP') && define('W7_CLOUD_SDK_DEVELOP', 0);

class Request {
	protected $apiUrl = 'http://api.w7.cc/';
	protected $isDevelop = W7_CLOUD_SDK_DEVELOP;
	protected $apiPath;
	protected $transToken;
	protected $cache;

	private $httpClient;
	private $requestContent;

	public function __construct(CacheInterface $cache = null) {
		$this->httpClient = new Client(array(
			'verify' => false,
			'http_errors' => true,
			'curl.options' => array(
				CURLOPT_SSLVERSION => CURL_SSLVERSION_TLSv1,
			),
		));

		$this->cache = $cache;
	}

	/**
	 * transttoken通过 site.token接口获取
	 * 请求接口时附加上此值，接口返回数据，都将以直接返回的方式，不会推送数据
	 * @param $transToken
	 * @return $this
	 */
	public function setTransToken($transToken) {
		$this->transToken = $transToken;
		return $this;
	}

	protected function post(array $data) {
		if (empty($this->apiPath)) {
			throw new \RuntimeException('接口地址不完整');
		}

		$header = array(
			'encode' => 'base64',
		);

		if ($this->isDevelop) {
			$header['User-Agent'] = 'we7test-develop';
		}

		if (!empty($this->transToken)) {
			$data['token'] = $this->transToken;
		}

		try {
			$response = $this->httpClient->post(sprintf('%s%s', $this->apiUrl, trim($this->apiPath, '/')), array(
				'form_params' => $data,
				'headers' => $header,
			));

			$content = $response->getBody()->getContents();
			$this->requestContent = $content;
			return $this->decode($data['method'], $content);
		} catch (RequestException $e) {
			$statusCode = $e->getResponse()->getStatusCode();
			$content = $e->getResponse()->getBody()->getContents();
			$message = json_decode($content, true);
            if ($statusCode == '501') {
                throw new SiteRegisteredException($message['error'], $statusCode);
            }
            if ($statusCode == '502') {
                throw new \Exception('502 Bad Gateway', $statusCode);
            }
			if (!is_null($message) && $message) {
				throw new \Exception($message['error'], $statusCode);
			}
			return $this->decode($data['method'], $content);
		}
	}

	/**
	 * 用于调式接口返回的数据
	 * @return mixed
	 */
	public function getRequestContent() {
		return $this->requestContent;
	}

	protected function decode($method, $response) {
		if (!empty($this->cache)) {
			return Shipping::instance()->decode($response, $this->cache->load($method));
		} else {
			return Shipping::instance()->decode($response, '');
		}
	}
}
